@@ -1,11 +1,11 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "git-timemachine" "4.13"
+(define-package "git-timemachine" "20250128.940"
   "Walk through git revisions of a file."
   '((emacs     "24.3")
     (transient "0.1.0"))
-  :url "https://gitlab.com/pidu/git-timemachine"
-  :commit "ac933e5cd29583c131401f3bd991d98129c316df"
-  :revdesc "ac933e5cd295"
+  :url "https://codeberg.org/pidu/git-timemachine"
+  :commit "d1346a76122595aeeb7ebb292765841c6cfd417b"
+  :revdesc "d1346a761225"
   :keywords '("vc")
   :authors '(("Peter Stiernström" . "peter@stiernstrom.se"))
   :maintainers '(("Peter Stiernström" . "peter@stiernstrom.se")))
